@@ -1,3 +1,4 @@
+from asyncio.coroutines import iscoroutinefunction
 from logging import error
 from typing import Dict
 import paho.mqtt.client as mqtt
@@ -198,11 +199,15 @@ class Module:
         return future.result()
 
     def on_method(self, name: str, cb):
-        def handler(topic: str, payload: bytes):
+        async def handler(topic: str, payload: bytes):
             [notused, rid] = topic.split("=")
             try:
                 data = json.loads(payload.decode())
-                result = cb(data)
+                if asyncio.iscoroutinefunction(cb):
+                    result = await cb(data)
+                else:
+                    result = cb(data)
+
                 self.publish_mqtt(
                     f"$iothub/methods/res/{200}/?$rid={rid}", json.dumps(result))
             except Exception as e:
@@ -247,8 +252,11 @@ class Module:
         if not self.client:
             raise error("Client not initialized")
 
-        def handler(c, u, message):
-            cb(message.topic, message.payload)
+        async def handler(c, u, message):
+            if asyncio.iscoroutinefunction(cb):
+                await cb(message.topic, message.payload)
+            else:
+                cb(message.topic, message.payload)
 
         self.client.subscribe(topic)
         self.topics.append(topic)
